/* cron-utils.c
 * SommerCampus 2006
 * Linux/Unix Systemprogrammierung
 *
 * Licence: GPL-2 */


#include "cron-utils.h"


int get_actual_cron_time(cron_time *ctime)
{
	time_t unix_time;
	struct tm *su_time;

	if(time(&unix_time) == -1)
		return 0;

	su_time = localtime(&unix_time);

	ctime->minute = su_time->tm_min;
	ctime->hour = su_time->tm_hour;
	ctime->day = su_time->tm_mday;
	ctime->month = su_time->tm_mon;

	return 1;
}


int match_cron_time(const cron_time *ctime1, const cron_time *ctime2)
{

	if( (ctime1->minute != ctime2->minute) &&
		(ctime1->minute != -1) &&
		(ctime2->minute != -1) )
		return 0;

	if( (ctime1->hour != ctime2->hour) &&
                (ctime1->hour != -1) &&
                (ctime2->hour != -1) )
                return 0;

	if( (ctime1->day != ctime2->day) &&
                (ctime1->day != -1) &&
                (ctime2->day != -1) )
                return 0;

	if( (ctime1->month != ctime2->month) &&
                (ctime1->month != -1) &&
                (ctime2->month != -1) )
                return 0;
	
	return 1;
}

int add_cron_entry(cron_entry **centry, const cron_entry *new)
{
	cron_entry *tmp, *head;
	size_t count;

	count = *centry ? (**centry).count : 0;

	tmp = realloc(*centry, sizeof(cron_entry) * (count + 1));

	if(!tmp)
		return 0;
	
	if(tmp != *centry)
		*centry = tmp;

	head = *centry;

	if(!count)
	{
		/* Erster Eintrag */
		head[0] = *new;
		head[0].count = 1;
		return 1;
	}

	head[head[0].count] = *new;
	head[0].count++;
	
	return 1;
}

int get_next_job(FILE *f, cron_entry *centry)
{
	char line[2048], tmp[2048], *real_line, *trim;
	size_t i,j,len, col, trim_counter = 0, trim_len;

	if(feof(f))
		return 0;

	if(ferror(f))
		return -1;


	if(fgets(line, 2048, f))
	{
		len = strlen(line);
		line[len - 1] = line[len - 1] != '\n' ? line[len - 1] : 0;
		len -= line[len - 1] == 0 ? 1 : 0;

		/* left trim */

		j=0;
		while(line[j] && isblank(line[j]))
			j++;

		real_line = line + j;

		if(j)
			len = strlen(real_line);
		
		col = j = 0;

		for(i=0; i < len; ++i)
		{
			if(real_line[0] == '#' || real_line[0] == 0)
				return 2;
			if(real_line[i] != ' ')
				tmp[j++] = real_line[i];
			else {
				tmp[j++] = 0;

				switch(col) {
					case 0:
						centry->ctime.minute = *tmp == '*' ? -1 : atoi(tmp);
						break;
					case 1:
						centry->ctime.hour = *tmp == '*' ? -1 : atoi(tmp);
						break;
					case 2:
						centry->ctime.day = *tmp == '*' ? -1 : atoi(tmp);
						break;
					case 3:
						centry->ctime.month = *tmp == '*' ? -1 : atoi(tmp);

						/* trim */

						trim = real_line + i + 1;
						trim_counter = 0;

						while(trim[trim_counter] && isblank(trim[trim_counter]))
							trim_counter++;

						trim += trim_counter;

						trim_len = strlen(trim) - 1;

						while(trim_len && isblank(trim[trim_len]))
							trim[trim_len--] = 0;

						strcpy(centry->command, trim);
						return 1;
				}
				*tmp = 0;
				col++;
				j=0;
			}
			
		}
		if(!len)
			return 2;
		
		return -1;
	} else {
		if(feof(f))
			return 0;

		if(ferror(f))
			return -1;
	}

	return -1;
}

char **get_users_from_spool(void)
{
	char **list=NULL, *tmp=NULL, **ltemp;
	DIR *dir;
	struct dirent* dirInfo;
	size_t len, listlen=0;

	dir = opendir(SPOOLER "/crontabs");

	if(!dir)
		return NULL;

	while( (dirInfo = readdir(dir)) )
	{
		if(strcmp(dirInfo->d_name, ".") && strcmp(dirInfo->d_name, ".."))
		{
			len = strlen(dirInfo->d_name);
			tmp = malloc(len + 1);
			
			if(!tmp)
			{
				free(list);
				return NULL;
			}

			strcpy(tmp, dirInfo->d_name);

			ltemp = realloc(list, sizeof(char*) * (listlen + 2));

			if(!ltemp)
			{
				free(tmp);
				/* FIXME: LEAK: man sollte alle andere auch freigeben */
				return NULL;
			}

			if(ltemp != list)
				list = ltemp;

			list[listlen] = tmp;
			list[listlen + 1] = NULL;

			listlen++;
		}
	}

	closedir(dir);

	return list;
}

void free_user_list(char **list)
{
	size_t i = 0;
	if(!list || !*list) return;

	while(list[i])
	{
		free(list[i++]);
	}

	free(list);
}

cron_entry *get_all_cron_jobs(void)
{
	size_t count = 0, i;
	char **ulist, path[1024];
	cron_entry *centry = NULL, currentjob;
	FILE *fp;

	/* get user list */

	ulist = get_users_from_spool();

	if(!ulist)
		return NULL;

	while(ulist[count])
	{
		sprintf(path, SPOOLER "/crontabs/%s", ulist[count]);

		fp = fopen(path, "r");

		if(fp)
		{
			while(1)
			{
				strcpy(currentjob.user, ulist[count]);

				i = get_next_job(fp, &currentjob);

				//printf("get_next_job = %d\n", i);

				

				if(i==0 || i==-1)
					break;

				if(i != 2)
					add_cron_entry(&centry, &currentjob);
			}
			
			
			fclose(fp);
		}

		count++;
	}

	free_user_list(ulist);

	return centry;	
}
