/* cron-utils.h
 * SommerCampus 2006
 * Linux/Unix Systemprogrammierung
 *
 * Licence: GPL-2 */

#ifndef CRON_UTILS_H
#define CRON_UTILS_H

#ifdef HAVE_CONFIG_H

#include <config.h>

#endif


#include <sys/stat.h>
#include <sys/types.h>
#include <dirent.h>
#include <pwd.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/wait.h>
#include <signal.h>
#include <syslog.h>

#include <time.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <ctype.h>

typedef struct {
	/* Wenn wert == -1 ==> wert entspricht '*' */
	int minute;     /* 0 - 59 */
	int hour;       /* 0 - 23 */
	int day;  	/* 1 - 31 */
	int month;      /* 1 - 12 */
} cron_time;


typedef struct {
	size_t count;		/* wesentlich nur im ersten Element */
	cron_time ctime;
	char user[32];
	char command[1024];
} cron_entry;


int get_actual_cron_time(cron_time *ctime);
/* return: 1 bei Erfolg, 0 sonst */

int match_cron_time(const cron_time *ctime1, const cron_time *ctime2);
/* return: 1 wenn beide Zeiten gleich sind, 0 sonst */

int add_cron_entry(cron_entry **centry, const cron_entry *new);
/* return: 1 bei Erfolg, 0 sonst
 * am Anfang muss *centry = NULL; */

int get_next_job(FILE *f, cron_entry *centry);
/* return: 1  Bei Erfolg
 * 	   0  f wurde vollstndig gelesen
 * 	   2  Zeile wird ignoriert
 * 	   -1 bei Fehlern */

char **get_users_from_spool(void);
/* liefert einen Zeiger auf Array von Benutzern, letzter ist NULL */

void free_user_list(char **list);

cron_entry *get_all_cron_jobs(void);
/* return: array mit den cronjobs (muss gefreet werden) 
 * Fehler: NULL */

#endif
